/*******************************************************************
 *
 *    DESCRIPTION:	ata_atapi.c
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

/** include files **/
#include "string.h"
#include "stdio.h"
#include "ata_if.h"

#include "timer.h"

/** local definitions **/

/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/


/** public functions **/

unsigned int ata_atapi_init( ata_ide_info_type *ata_ide_info_trans )
{
	unsigned int  		ata_result = 0;
	unsigned int		ata_data[512];
	unsigned int		tmp_counter;
	ata_ide_info_type   ata_ide_info;
	ATAPI_IDENTIFY_RESULT	atapi_identify_result; 

	memset( &ata_ide_info, 0x00, sizeof(ata_ide_info) );

	ata_result = ata_ide_execute_drive_diagnose( ATA_MASTER_DRIVE );
	if( ata_result & ATA_ERROR )
	{
		ata_result = ata_io_read_error( ATA_MASTER_DRIVE );

		return( ATA_ERROR | ata_result );
	}

	if( (ata_result&0x0F) == 0x01 )
	{	
		ata_result = ata_atapi_identify_device( ATA_MASTER_DRIVE, (unsigned int *)&ata_data );
		if( ata_result & ATA_ERROR )
		{
			ata_result = ata_io_read_error( ATA_MASTER_DRIVE );

			return( ATA_ERROR | ata_result );
		}

		memcpy( &atapi_identify_result, &ata_data, sizeof(atapi_identify_result) );

		ata_ide_info_trans->master_atapi.available					= ATA_AVAILABLE;
		ata_ide_info_trans->master_atapi.capabilities				= atapi_identify_result.capabilities;
		ata_ide_info_trans->master_atapi.configuration				= atapi_identify_result.configuration;

		for( tmp_counter = 0; tmp_counter < sizeof( atapi_identify_result.model_name );  )
		{
			ata_ide_info_trans->master_atapi.model_name[tmp_counter]   = atapi_identify_result.model_name[tmp_counter+1];
			ata_ide_info_trans->master_atapi.model_name[tmp_counter+1] = atapi_identify_result.model_name[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info_trans->master_atapi.model_name[sizeof(atapi_identify_result.model_name)+1] = 0x00;

		for( tmp_counter = 0; tmp_counter < sizeof( atapi_identify_result.serial_number ); )
		{
			ata_ide_info_trans->master_atapi.model_serial_number[tmp_counter]   = atapi_identify_result.serial_number[tmp_counter+1];
			ata_ide_info_trans->master_atapi.model_serial_number[tmp_counter+1] = atapi_identify_result.serial_number[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info_trans->master_atapi.model_serial_number[sizeof(atapi_identify_result.serial_number)+1] = 0x00;

		for( tmp_counter = 0; tmp_counter < sizeof( atapi_identify_result.firmware_revision ); )
		{
			ata_ide_info_trans->master_atapi.firmware_revision[tmp_counter]   = atapi_identify_result.firmware_revision[tmp_counter+1];
			ata_ide_info_trans->master_atapi.firmware_revision[tmp_counter+1] = atapi_identify_result.firmware_revision[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info_trans->master_atapi.firmware_revision[sizeof(atapi_identify_result.firmware_revision)+1] = 0x00;
		ata_ide_info_trans->master_type = ATAPI_TYPE;
	}
	return( ATA_OK );
}


unsigned int ata_atapi_execute_drive_diagnose( unsigned int drive )
{
	return( ata_ide_execute_drive_diagnose( drive ) );
}


unsigned int ata_atapi_identify_device( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;

	drive = drive;
	ata_io_write_command( drive, ATAPI_CMD_IDENT_DRIVE );

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	ata_result = ata_io_read_status( drive );

	if( ata_result && ATA_STATUS_DRQ )
	{
		for( ata_counter = 0; ata_counter < 256; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATAPI_DATA_REG, ata_data );
			ata_data++;
		}
		return( ATA_OK | ata_result );
	}	
	return( ATA_ERROR | ata_result );
}

void ata_atapi_select_device( unsigned int drive )
{
	while (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)){;};  //Wait until Device is ready	
	ata_io_write_byte( drive, ATA_DRIVE_HEAD_REG, 0x0a0 );
	while (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)){;};  //Wait until Device is ready
}


unsigned int ata_atapi_reset_device( unsigned int drive )
{
	unsigned int ata_result;
	unsigned int sense_key;
	unsigned int asc;
	unsigned int asc_q;

	// setup Feature register
	ata_io_write_byte( drive, ATAPI_FEATURE_REG, 0x0E );

	timer_delay_micro_seconds( 100 );

	// setup Feature register
	ata_io_write_byte( drive, ATAPI_FEATURE_REG, 0x0A );

	ata_result = ata_io_read_status( drive );

	while( 1 )
	{
		ata_result = ata_io_read_status( drive );
		if( (ata_result & ATA_STATUS_BSY) == 0 )
		{
			break;
		}
	}  //Wait until Device is ready	

	ata_atapi_select_device( drive );

	while( ata_io_read_status( drive ) & ATA_STATUS_ERR )
	{
		ata_atapi_get_sense( drive, &sense_key, &asc, &asc_q );
		if( (sense_key == 0x70) || (sense_key == 0x71) )
		{
			return( ATA_ERROR );
		} 
	};  //Wait until Device is ready

	return( ATA_OK );
}


unsigned int ata_atap_set_slow_speed( unsigned int drive )
{
	unsigned int buffer[32];
	unsigned int ata_result = 0;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xBB;	// command
	buffer[2] = 0x01;	// 
	buffer[3] = 0x60;	// 

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	ata_result = ata_io_read_status( drive );
	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );
		printf("error\n");
		return( ATA_ERROR | ata_result );
	}
	return( ATA_OK );
}


unsigned int ata_atapi_initialize_device_parameter( unsigned int drive )
{
	drive = drive;
	return( ATA_ERROR );
}


unsigned int ata_atapi_read_sector( unsigned int drive, unsigned long lba, unsigned long sector_count, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xBE;	// command
	buffer[1] = 0x00;	// 
	buffer[2] = ((lba>>24) & 0x00FF);			//  LBA msb
	buffer[3] = ((lba>>16) & 0x00FF);			// 
	buffer[4] = ((lba>>8) & 0x00FF);			// 
	buffer[5] = (lba & 0x00FF);	   	    		// LBA lsb
	buffer[6] = ((sector_count>>(unsigned long)16) & (unsigned long)0x00FF);	// Transfer Length msb
	buffer[7] = ((sector_count>>(unsigned long)8) & (unsigned long)0x00FF);	//
	buffer[8] = (sector_count & (unsigned long)0x00FF);	    // Transfer Length lsb
	buffer[9] = 0x10;	// Reserved
	buffer[11] = 0x00;	// 

	ata_result = ata_io_read_status( drive );

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		if( byte_count & 0x0001 )
		{
			byte_count++;
		}

		word_count = byte_count >> 1;

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
		ata_result = ata_io_read_status( drive );
	}
	ata_result = ata_io_read_status( drive );
	return( ATA_OK | ata_result  );
}


unsigned int ata_atapi_drive_door( unsigned int drive, unsigned int action )
{
	unsigned int buffer[32];
	unsigned int ata_result = 0;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x1B;	// command
	buffer[1] = 0x01;	// 
	buffer[4] = action;	// 

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	ata_result = ata_io_read_status( drive );
	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );
		printf("error\n");
		return( ATA_ERROR | ata_result );
	}
	return( ATA_OK );
}

unsigned int ata_atapi_get_status( unsigned int drive, unsigned int *sense_key, unsigned int *asc, unsigned int *asc_q )
{
	unsigned int buffer[32];
	unsigned int ata_result = 0;

	*sense_key = 0x00;
	*asc = 0x00;
	*asc_q = 0x00;

	memset( buffer, 0x00, sizeof(buffer) );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	while( 1 )
	{
		ata_result = ata_io_read_status( drive );
		
		// wait for busy to be released
		while( ata_result & ATA_STATUS_BSY )
		{
			ata_result = ata_io_read_status( drive );
		}

		// no data to transfered
		if( (ata_result & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0x00 )
		{
			break;
		}
		
		// no data to transfered
		if( ata_result & ATA_STATUS_ERR )
		{
			break;
		}
	}		

	if( (ata_result & ATA_STATUS_ERR) == 0x00 )
	{
		*sense_key = 0x00;
		*asc = 0x00;
		*asc_q = 0x00;
		return( ATA_OK | ata_result );
	}
	
	ata_atapi_get_sense( drive, sense_key, asc, asc_q );

	return( ATA_ERROR );
}


unsigned int ata_atapi_get_sense( unsigned int drive, unsigned int *sense_key, unsigned int *asc, unsigned int *asc_q )
{
	unsigned int buffer[32];
	unsigned int ata_counter;
	unsigned int ata_result = 0;
	unsigned char byte_count;
	unsigned int  ata_data[256];

	*sense_key = 0x00;
	*asc = 0x00;
	*asc_q = 0x00;

	memset( ata_data, 0x00, sizeof(ata_data) );
	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x03;	// command
	buffer[4] = 0xFF;	// 

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	while( 1 )
	{
		ata_result = ata_io_read_status( drive );
		
		// wait for busy to be released
		while( ata_result & ATA_STATUS_BSY )
		{
			ata_result = ata_io_read_status( drive );
		}
		
		// no data to transfered
		if( (ata_result & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0x00 )
		{
			return( ATA_ERROR | ata_result );
		}

		//there is data to be get
		if( (ata_result & ATA_STATUS_DRQ) )
		{
			break;
		}
	}		
	// read byte count register low
	ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count );

	for( ata_counter = 0; ata_counter < (byte_count/2); ata_counter++ )
	{
		ata_result = ata_io_read_word( drive, ATA_DATA_REG, &ata_data[ata_counter] );
	}
	if( (ata_data[0] == 0x70) || (ata_data[0] == 0x71) )
	{
		*sense_key = (ata_data[1] & 0x0F);
		*asc	   = (ata_data[6] & 0x0FF);
		*asc_q	   = (ata_data[6]>>8);
	}
	return( ATA_OK | ata_result );
}

unsigned int ata_atapi_get_mech_status( unsigned int drive, unsigned int *mech_status )
{
	unsigned int buffer[32];
	unsigned int ata_counter;
	unsigned int ata_result = 0;
	unsigned int byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	unsigned int  ata_error;
	unsigned int  ata_data[2048];


	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xBD;	// command
	buffer[8] = 0x08;	// 

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
		if( ata_result & ATA_STATUS_ERR )
		{
			ata_error = ata_io_read_error( drive );
			printf( "error %0X\n", ata_error );
		}
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	// read byte count register low
	ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

	// read byte count reguster high
	ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

	byte_count = (byte_count_h<<8) + byte_count_l;

	word_count = byte_count >> 1;

	for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
	{
		ata_result = ata_io_read_word( drive, ATA_DATA_REG, &ata_data[ata_counter] );
		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );
			printf("error\n");
			return( ATA_ERROR | ata_result );
		}
	}
	memcpy( mech_status, &ata_data[0], byte_count );
	return( ATA_OK | ata_result );
}


unsigned int ata_atapi_read_sector_lba( unsigned int drive, unsigned long lba, unsigned long sector_count, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xA8;	// command
	buffer[1] = 0x08;	// 
	buffer[2] = ((lba>>24) & 0x00FF);			//  LBA msb
	buffer[3] = ((lba>>16) & 0x00FF);			// 
	buffer[4] = ((lba>>8) & 0x00FF);			// 
	buffer[5] = (lba & 0x00FF);	   	    		// LBA lsb
	buffer[6] = ((sector_count>>(unsigned long)24) & (unsigned long)0x00FF);	// Transfer Length msb
	buffer[7] = ((sector_count>>(unsigned long)16) & (unsigned long)0x00FF);	//
	buffer[8] = ((sector_count>>(unsigned long)8) & (unsigned long)0x00FF);	//
	buffer[9] = (sector_count & (unsigned long)0x00FF);	    // Transfer Length lsb
	buffer[10] = 0x00;	// Reserved
	buffer[11] = 0x00;	// 

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		if( byte_count & 0x0001 )
		{
			byte_count++;
		}

		word_count = byte_count >> 1;

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
		ata_result = ata_io_read_status( drive );
	}
	ata_result = ata_io_read_status( drive );
	return( ATA_OK | ata_result  );
}

unsigned int ata_atapi_read_TOC( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x43;	// command
	buffer[1] = 0x02;	// MSF Format
	buffer[2] = 0x02;	// Full TOC
	buffer[7] = 0x08;	//
	buffer[8] = 0x00;	//
	buffer[9] = 0x00;	// 


	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		word_count = (byte_count >> 1) + 1;

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
	}
	return( ATA_OK | ata_result  );
}

unsigned int ata_atapi_read_CD_TEXT( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x43;	// command
	buffer[1] = 0x02;	// MSF Format
	buffer[2] = 0x05;	// Full TOC
	buffer[7] = 0x08;	//
	buffer[8] = 0x00;	//
	buffer[9] = 0x00;	// 

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		if( byte_count & 0x0001 )
		{
			byte_count++;
		}

		word_count = (byte_count >> 1);

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
	}
	return( ATA_OK | ata_result  );
}

unsigned int ata_atapi_read_DVD_structure( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xAD;	// command
	buffer[8] = 0x08;	//
	buffer[9] = 0x00;	//

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		if( byte_count & 0x0001 )
		{
			byte_count++;
		}

		word_count = byte_count >> 1;

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
	}
	return( ATA_OK | ata_result  );
}


unsigned int ata_atapi_read_current_position( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  buffer[12];
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  byte_count;
	unsigned char byte_count_l;
	unsigned char byte_count_h;
	unsigned int  word_count;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x42;	// command
	buffer[1] = 0x02;	//
	buffer[2] = 0x40;	//
	buffer[3] = 0x01;	//

	buffer[7] = 0x08;	//
	buffer[8] = 0x00;	//
	buffer[9] = 0x00;	// 


	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	while( (ata_io_read_status( drive ) & ATA_STATUS_BSY));

	if( (ata_io_read_status( drive ) & (ATA_STATUS_BSY | ATA_STATUS_DRQ)) == 0 )
	{
		printf( "no data\n");
		ata_result = ata_io_read_status( drive );
		return( ATA_ERROR | ata_result );
	}

	if( ata_io_read_status( drive ) & ATA_STATUS_DRQ )
	{
		// read byte count register low
		ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_L_REG, &byte_count_l );

		// read byte count reguster high
	    ata_result = ata_io_read_byte( drive, ATAPI_BYTE_COUNT_H_REG, &byte_count_h );

		byte_count = (byte_count_h<<8) + byte_count_l;

		if( byte_count & 0x0001 )
		{
			byte_count++;
		}

		word_count = byte_count >> 1;

		for( ata_counter = 0; ata_counter < word_count; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
	}
	return( ATA_OK | ata_result  );
}

// Audio CD functions
unsigned int ata_atapi_start_scan( unsigned int drive, unsigned int direction, unsigned long start_time )
{
	unsigned int buffer[32];
	unsigned int ata_result = 0;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0xBA;	// command
	if( direction == ATAPI_SCAN_FORWARD )
	{
		buffer[1] = 0x00; //forward
	}
	else
	{
		buffer[1] = 0x10; //rewind
	}

	buffer[3] = ((start_time>>16) & 0x00FF);			// 
	buffer[4] = ((start_time>>8) & 0x00FF);			// 
	buffer[5] = (start_time & 0x00FF);	   	    		// LBA lsb

	buffer[9] = 0x040;//MSF type

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	ata_result = ata_io_read_status( drive );
	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );
		printf("error\n");
		return( ATA_ERROR | ata_result );
	}
	return( ATA_OK );
}


unsigned int ata_atapi_play_cd( unsigned int drive, unsigned long start_time, unsigned long end_time )
{
	unsigned int buffer[32];
	unsigned int ata_result = 0;

	memset( buffer, 0x00, sizeof(buffer) );
	// setup packet buffer
	buffer[0] = 0x47;	// command

	buffer[3] = ((start_time>>16) & 0x00FF);			// 
	buffer[4] = ((start_time>>8) & 0x00FF);			// 
	buffer[5] = (start_time & 0x00FF);	   	    		// LBA lsb

	buffer[6] = ((end_time>>16) & 0x00FF);			// 
	buffer[7] = ((end_time>>8) & 0x00FF);			// 
	buffer[8] = (end_time & 0x00FF);	   	    		// LBA lsb

	ata_atapi_select_device( drive );

 	ata_atapi_send_packet_command( drive, &buffer[0] );

	// wait for BSY to released
	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) == 1  );

	ata_result = ata_io_read_status( drive );
	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );
		printf("error\n");
		return( ATA_ERROR | ata_result );
	}
	return( ATA_OK );
}

/** private functions **/

unsigned int ata_atapi_send_packet_command( unsigned int drive, unsigned int *command_buffer )
{
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;
	unsigned int  tmp_word;

	// setup Feature register
	ata_io_write_byte( drive, ATAPI_FEATURE_REG, 0x0A );

	// setup Sector Count register
	ata_io_write_byte( drive, ATAPI_SECTOR_COUNT_REG, 0x00 );

	// setup Sector Number register
	ata_io_write_byte( drive, ATAPI_SECTOR_NR_REG, 0x00 );

	// setup byte count register low
	ata_io_write_byte( drive, ATAPI_BYTE_COUNT_L_REG, 0x00 );

	// setup byte count reguster high
    ata_io_write_byte( drive, ATAPI_BYTE_COUNT_H_REG, 0x08 );

	// write head number
	ata_io_write_byte( drive, ATA_DRIVE_HEAD_REG, 0xA0 );

	// issue PACKET COMMAND
	ata_io_write_command( drive, ATAPI_CMD_PACKET );

	// wait for DRQ
	while( (ata_io_read_status( drive ) & ATA_STATUS_DRQ) == 0  );

	for( ata_counter = 0; ata_counter < 6; ata_counter++ )
	{
		tmp_word = (command_buffer[(2*ata_counter)+1]<<8) + command_buffer[(2*ata_counter)];
		ata_io_write_word( drive, ATA_DATA_REG, tmp_word );
	}
	return( ATA_OK );
}