/*******************************************************************
 *
 *    DESCRIPTION:	ata_ide.c
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

/** include files **/
#include "string.h"
#include "stdio.h"
#include "ata_if.h"


/** local definitions **/

/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/


/** public functions **/

unsigned int ata_ide_init( ata_ide_info_type *ata_ide_info_trans )
{
	unsigned int  		ata_result = 0;
	unsigned int		ata_data[512];
	unsigned int		tmp_counter;
	ata_ide_info_type   ata_ide_info;
	ATA_IDENTIFY_RESULT	ata_identify_result; 

	memset( &ata_ide_info, 0x00, sizeof(ata_ide_info) );

	ata_result = ata_ide_execute_drive_diagnose( ATA_MASTER_DRIVE );
	if( ata_result & ATA_ERROR )
	{
		ata_result = ata_io_read_error( ATA_MASTER_DRIVE );

		return( ATA_ERROR | ata_result );
	}

	if( (ata_result&0x0F) == 0x01 )
	{	
		ata_result = ata_ide_identify_device( ATA_MASTER_DRIVE, (unsigned int *)&ata_data );
		if( ata_result & ATA_ERROR )
		{
			ata_result = ata_io_read_error( ATA_MASTER_DRIVE );

			return( ATA_ERROR | ata_result );
		}

		memcpy( &ata_identify_result, &ata_data, sizeof(ata_identify_result) );

		ata_ide_info.master.available					= ATA_AVAILABLE;
		ata_ide_info.master.number_of_heads 			= ata_identify_result.number_of_heads;
		ata_ide_info.master.number_of_cylinders 		= ata_identify_result.number_of_cylinders;
		ata_ide_info.master.number_of_sectors			= (unsigned long)ata_identify_result.number_of_sectors[0] + ((unsigned long)ata_identify_result.number_of_sectors[1] << 16);
		ata_ide_info.master.number_of_sectors_per_track = ata_identify_result.sectors_per_track;
		ata_ide_info.master.bytes_per_sector			= 512;
		ata_ide_info.master.capabilities				= ata_identify_result.capabilities;
		ata_ide_info.master.configuration				= ata_identify_result.configuration;

		for( tmp_counter = 0; tmp_counter < sizeof( ata_identify_result.model_name );  )
		{
			ata_ide_info.master.model_name[tmp_counter]   = ata_identify_result.model_name[tmp_counter+1];
			ata_ide_info.master.model_name[tmp_counter+1] = ata_identify_result.model_name[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info.master.model_name[20] = 0x00;

		for( tmp_counter = 0; tmp_counter < sizeof( ata_identify_result.serial_number ); )
		{
			ata_ide_info.master.model_serial_number[tmp_counter]   = ata_identify_result.serial_number[tmp_counter+1];
			ata_ide_info.master.model_serial_number[tmp_counter+1] = ata_identify_result.serial_number[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info.master.model_serial_number[10] = 0x00;

		for( tmp_counter = 0; tmp_counter < sizeof( ata_identify_result.firmware_revision ); )
		{
			ata_ide_info.master.firmware_revision[tmp_counter]   = ata_identify_result.firmware_revision[tmp_counter+1];
			ata_ide_info.master.firmware_revision[tmp_counter+1] = ata_identify_result.firmware_revision[tmp_counter];
			tmp_counter = tmp_counter + 2;
		}
		ata_ide_info.master.firmware_revision[8] = 0x00;
	}
	memcpy( ata_ide_info_trans, &ata_ide_info, sizeof(ata_ide_info) );
	ata_ide_info_trans->master_type = ATAPI_TYPE;

	return( ATA_OK | ata_result );
}


unsigned int ata_ide_execute_drive_diagnose( unsigned int drive )
{
	unsigned int  ata_result = 0;

	drive = drive;

	ata_io_write_byte( drive, ATA_COMMAND_REG, ATA_CMD_EXEC_DRIVE_DIAG );

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );
	
	ata_result = ata_io_read_error( drive );
	
	return( ata_result );
}


unsigned int ata_ide_identify_device( unsigned int drive, unsigned int *ata_data )
{
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;

	drive = drive;
	ata_io_write_command( drive, ATA_CMD_IDENT_DRIVE );

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	ata_result = ata_io_read_status( drive );

	if( ata_result && ATA_STATUS_DRQ )
	{
		for( ata_counter = 0; ata_counter < 256; ata_counter++ )
		{
			ata_result = ata_io_read_word( drive, ATA_DATA_REG, ata_data );
			ata_data++;
		}
		return( ATA_OK | ata_result );
	}	
	return( ATA_ERROR | ata_result );
}


unsigned int ata_ide_initialize_device_parameter( unsigned int drive )
{
	drive = drive;
	return( ATA_ERROR );
}


unsigned int ata_ide_read_sector( unsigned int drive, unsigned int sector, unsigned int cylinder, unsigned int head, unsigned int sector_count, unsigned int *ata_data )
{
	unsigned int  ata_counter;
	unsigned int  sector_counter;
	unsigned int  ata_result = 0;

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_DRIVE_HEAD_REG, ((head & 0x000F) | 0x00A0) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while(  ((ata_result & ATA_STATUS_BSY) != 0) && ((ata_result & ATA_STATUS_DRQ) == 0) );

	// write sector number
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_NUMBER_REG, (sector & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write cylinder number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_REG, (cylinder & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_1_REG, ((cylinder>>8) & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write number of sectors
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_COUNT_REG, (sector_count & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write read sector command
	ata_result = ata_io_write_command( drive, ATA_CMD_READ_SECTORS );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// wait for DRQ
	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while( ((ata_result & ATA_STATUS_DRQ) != ATA_STATUS_DRQ) );

	// read data
	for( sector_counter = 0; sector_counter	< sector_count; sector_counter++ )
	{
		for( ata_counter = 0; ata_counter < 256; ata_counter++ )
		{
			ata_result = ata_io_read_word( ATA_MASTER_DRIVE, ATA_DATA_REG, ata_data );

			if( ata_result & ATA_STATUS_ERR )
			{
				ata_result = ata_io_read_error( drive );

				return( ATA_ERROR | ata_result );
			}

			ata_data++;
		}
	}

	return( ATA_OK | ata_result );
}


unsigned int ata_ide_read_sector_without_retries( unsigned int drive, unsigned int sector, unsigned int cylinder, unsigned int head, unsigned int sector_count, unsigned int *ata_data )
{
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_DRIVE_HEAD_REG, ((head & 0x000F) | 0x00A0) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while(  ((ata_result & ATA_STATUS_BSY) != 0) && ((ata_result & ATA_STATUS_DRQ) == 0) );

	// write sector number
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_NUMBER_REG, (sector & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write cylinder number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_REG, (cylinder & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_1_REG, ((cylinder>>8) & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write number of sectors
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_COUNT_REG, (sector_count & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write read sector command
	ata_result = ata_io_write_command( drive, ATA_CMD_READ_SECTORS_NO_VER );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// wait for DRQ
	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while( ((ata_result & ATA_STATUS_DRQ) != ATA_STATUS_DRQ) );

	// read data
	for( ata_counter = 0; ata_counter < 256; ata_counter++ )
	{
		ata_result = ata_io_read_word( ATA_MASTER_DRIVE, ATA_DATA_REG, ata_data );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}

		ata_data++;
	}

	return( ATA_OK | ata_result );
}

unsigned int ata_ide_read_sector_lba( unsigned int drive, unsigned long lba, unsigned int sector_count, unsigned int *ata_data )
{
	unsigned int  ata_counter;
	unsigned int  ata_result = 0;

	do
	{
		ata_result = ata_io_read_status( drive );
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_DRIVE_HEAD_REG, ((lba >>24)&0x0FL) | 0xE0 );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while(  ((ata_result & ATA_STATUS_BSY) != 0) && ((ata_result & ATA_STATUS_DRQ) == 0) );

	// write sector number
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_NUMBER_REG, (lba&0xFFL) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write cylinder number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_REG, ((lba >>8L)&0xFFL) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write head number
	ata_result = ata_io_write_byte( drive, ATA_CYLINDER_1_REG, ((lba >>16L)&0xFFL) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write number of sectors
	ata_result = ata_io_write_byte( drive, ATA_SECTOR_COUNT_REG, (sector_count & 0x00FF) );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// write read sector command
	ata_result = ata_io_write_command( drive, ATA_CMD_READ_SECTORS );

	if( ata_result & ATA_STATUS_ERR )
	{
		ata_result = ata_io_read_error( drive );

		return( ATA_ERROR | ata_result );
	}

	// wait for DRQ
	do
	{
		ata_result = ata_io_read_status( drive );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}
	}
	while( ((ata_result & ATA_STATUS_DRQ) != ATA_STATUS_DRQ) );

	// read data
	for( ata_counter = 0; ata_counter < 256; ata_counter++ )
	{
		ata_result = ata_io_read_word( ATA_MASTER_DRIVE, ATA_DATA_REG, ata_data );

		if( ata_result & ATA_STATUS_ERR )
		{
			ata_result = ata_io_read_error( drive );

			return( ATA_ERROR | ata_result );
		}

		ata_data++;
	}

	return( ATA_OK | ata_result );
}


unsigned int ata_ide_read_verify_sectors( unsigned int drive )
{
	drive = drive;
	return( ATA_ERROR );
}


unsigned int ata_ide_write_sector( unsigned int drive )
{
	drive = drive;
	return( ATA_ERROR );
}


unsigned int ata_ide_write_sector_without_retries( unsigned int drive )
{
	drive = drive;
	return( ATA_ERROR );
}

/** private functions **/
