/*******************************************************************
 *
 *    DESCRIPTION:	ata_io.c
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

/** include files **/
#include "REG165.H"
#include "timer.h"
#include "io_if.h"
#include "ata_if.h"

/** local definitions **/

/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/
#define		ATA_WR_ACTIVE		0
#define		ATA_WR_NOT_ACTIVE	1

#define		ATA_RD_ACTIVE		0
#define		ATA_RD_NOT_ACTIVE	1

#define		ATA_RESET_ACTIVE		0
#define		ATA_RESET_NOT_ACTIVE	1
		
#define		ATA_OE_ENABLE	0					//		0 = enable,  buffers active
#define		ATA_OE_DISABLE	1					//		1 = disable, buffers in tri-state

#define		ATA_DIR_INPUT	0					// 		0 = input,  to C167
#define		ATA_DIR_OUTPUT	1					//		1 = output, to drive

#define		ATA_DATA_INPUT	0x00
#define		ATA_DATA_OUTPUT	0xFF

#define		ATA_DATA_MASK	0xFF
#define		ATA_DATA_LOW	0x00

#define		ATA_P2_MASK   	0x0FFF
#define		ATA_P2_SET   	0x0478

#define		ATA_DP2_MASK   	0x0FFF
#define		ATA_DP2_SET   	0x0CFF

#define		ATA_ODP2_MASK   0x0400
#define		ATA_ODP2_SET   	0x0400

/** public functions **/

unsigned int ata_io_init( void )
{
	unsigned long time_out_counter;
	unsigned int  ata_result = 0;

// setup ATA control lines
	_bfld( P2,   ATA_P2_MASK,   ATA_P2_SET );
	_bfld( DP2,  ATA_DP2_MASK,  ATA_DP2_SET );
	_bfld( ODP2, ATA_ODP2_MASK, ATA_ODP2_SET );

// setup ATA data lines to input
	_bfld( ATA_DATA_L, ATA_DATA_MASK, ATA_DATA_LOW );	
	_bfld( ATA_DATA_H, ATA_DATA_MASK, ATA_DATA_LOW );	
	_bfld( ATA_DIR_L,  ATA_DATA_MASK, ATA_DATA_INPUT );	
	_bfld( ATA_DIR_H,  ATA_DATA_MASK, ATA_DATA_INPUT );	

    IDE_POWER			= IDE_POWER_ON;

	ATA_RESET = ATA_RESET_ACTIVE;		// reset ATA drives
	timer_delay_milli_seconds( 500 );					// wait for 0,5 seconds
	ATA_RESET = ATA_RESET_NOT_ACTIVE;	// release reset

	timer_delay_milli_seconds( 1 );	// delay of 1ms to let drive set the BUSY flag

	time_out_counter = 0;
	do					// wait for drive has finished the power-up / reset
	{
		ata_result = ata_io_read_status( ATA_MASTER_DRIVE );
		timer_delay_milli_seconds( 1 );		// wait for 1 milliseconds
		time_out_counter++;
		if( time_out_counter > 0x8FFFFL )
		{
			return( ATA_ERROR_TIME_OUT );
		}
	}
	while(  (ata_result & ATA_STATUS_BSY) != 0  );

	return( ATA_OK | ata_result);
}


unsigned int ata_io_read_byte( unsigned int drive, unsigned int addr, unsigned char *data )
{
	drive = drive;

	ATA_OE	= ATA_OE_DISABLE;
	ATA_DIR = ATA_DIR_INPUT;

	P2 = (P2 & ~(unsigned int)ATA_REG_CMD_MASK) | (addr & ATA_REG_CMD_MASK);	

	_bfld( ATA_DATA_L, ATA_DATA_MASK, ATA_DATA_LOW );	
	_bfld( ATA_DATA_H, ATA_DATA_MASK, ATA_DATA_LOW );
		
	_bfld( ATA_DIR_L,  ATA_DATA_MASK, ATA_DATA_INPUT );	
	_bfld( ATA_DIR_H,  ATA_DATA_MASK, ATA_DATA_INPUT );	

	ATA_OE	= ATA_OE_ENABLE;

	_nop();
	_nop();

	ATA_RD	= ATA_RD_ACTIVE;
	
	_nop();	// delay about 200ns
	_nop();
	_nop();
	_nop();

	*data =  (unsigned char)ATA_DATA_L;

	_nop();
	_nop();

	ATA_RD	= ATA_RD_NOT_ACTIVE;

	return( ATA_OK );
}

unsigned int ata_io_write_byte( unsigned int drive, unsigned int addr, unsigned char data )
{
	drive = drive;

	ATA_OE	= ATA_OE_DISABLE;
	ATA_DIR = ATA_DIR_OUTPUT;

	P2 = (P2 & ~(unsigned int)ATA_REG_CMD_MASK) | (addr & ATA_REG_CMD_MASK);	

	ATA_DATA_L = (unsigned char)(data & 0x00FF);	
	_bfld( ATA_DATA_H, ATA_DATA_MASK, ATA_DATA_LOW );	

	_bfld( ATA_DIR_L,  ATA_DATA_MASK, ATA_DATA_OUTPUT );	
	_bfld( ATA_DIR_H,  ATA_DATA_MASK, ATA_DATA_OUTPUT );	

	ATA_OE	= ATA_OE_ENABLE;

	_nop();
	_nop();
	_nop();
	_nop();

	ATA_WR	= ATA_WR_ACTIVE;
	
	_nop();	// delay about 200ns
	_nop();
	_nop();
	_nop();
	
	ATA_WR	= ATA_WR_NOT_ACTIVE;

	return( ATA_OK );
}


unsigned int ata_io_read_word( unsigned int drive, unsigned int addr, unsigned int *data )
{
	drive = drive;

	ATA_OE	= ATA_OE_DISABLE;
	ATA_DIR = ATA_DIR_INPUT;

	P2 = (P2 & ~(unsigned int)ATA_REG_CMD_MASK) | (addr & ATA_REG_CMD_MASK);	

	_bfld( ATA_DATA_L, ATA_DATA_MASK, ATA_DATA_LOW );	
	_bfld( ATA_DATA_H, ATA_DATA_MASK, ATA_DATA_LOW );
		
	_bfld( ATA_DIR_L,  ATA_DATA_MASK, ATA_DATA_INPUT );	
	_bfld( ATA_DIR_H,  ATA_DATA_MASK, ATA_DATA_INPUT );	

	ATA_OE	= ATA_OE_ENABLE;

	_nop();	// delay about 200ns
	_nop();
	_nop();	// delay about 200ns
	_nop();

	ATA_RD	= ATA_RD_ACTIVE;

	_nop();	// delay about 200ns
	_nop();
	_nop();	// delay about 200ns
	_nop();

	*data =  (ATA_DATA_H << 8) + ATA_DATA_L;

	_nop();	// delay about 200ns
	_nop();

	ATA_RD	= ATA_RD_NOT_ACTIVE;

	return( ATA_OK );
}

unsigned int ata_io_write_word( unsigned int drive, unsigned int addr, unsigned int data )
{

	drive = drive;

	ATA_OE	= ATA_OE_DISABLE;
	ATA_DIR = ATA_DIR_OUTPUT;

	P2 = (P2 & ~(unsigned int)ATA_REG_CMD_MASK) | (addr & ATA_REG_CMD_MASK);	

	ATA_DATA_L = (unsigned char)(data & 0x00FF);	
	ATA_DATA_H = (unsigned char)((data & 0xFF00)>>8);	

	_bfld( ATA_DIR_L,  ATA_DATA_MASK, ATA_DATA_OUTPUT );	
	_bfld( ATA_DIR_H,  ATA_DATA_MASK, ATA_DATA_OUTPUT );	

	ATA_OE	= ATA_OE_ENABLE;

	ATA_WR	= ATA_WR_ACTIVE;
	
	_nop();	// delay about 200ns
	_nop();
	_nop();	// delay about 200ns
	_nop();
	
	ATA_WR	= ATA_WR_NOT_ACTIVE;

	return( ATA_OK );
}


unsigned int ata_io_write_command( unsigned int drive, unsigned int command )
{
	ata_io_write_byte( drive, ATA_COMMAND_REG, command );

	return( ATA_OK );
}


unsigned int ata_io_read_status( unsigned int drive )
{
	unsigned char result;

	ata_io_read_byte( drive, ATA_STATUS_REG, &result );

	return( result );
}


unsigned int ata_io_read_error( unsigned int drive )
{
	unsigned char result;

	ata_io_read_byte( drive, ATA_ERROR_REG, &result );

	return( result );
}
