/*******************************************************************
 *
 *    DESCRIPTION:	ata_mbr.c
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

/** include files **/
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "ata_if.h"

/** local definitions **/

/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/
static	BS_AND_BPB			ata_bs_and_bpb;

/** public functions **/

unsigned int ata_mbr_init( ata_ide_info_type *ata_trans )
{
	unsigned int		ata_result = 0;
	unsigned int		*ata_data;
	unsigned int		tmp_counter;


	ata_data = (unsigned int *)malloc( (unsigned int)ata_trans->master.bytes_per_sector );
	if( ata_data == NULL )
	{
		return( ATA_MALLLOC_ERROR | ata_result );
	}
	ata_result = ata_ide_read_sector( ATA_MASTER_DRIVE, ATA_BS_AND_BPB_SECTOR, 0, 0, 1, ata_data );

#ifdef DEBUG
	printf("ata_ide_read_sector(MBR) result:%04X \n", ata_result );
#endif

	memcpy( &ata_trans->master_mbr, ata_data, sizeof(ata_trans->master_mbr) );

	free( ata_data );

	for( tmp_counter = 0; tmp_counter<4; tmp_counter++ )
	{
		if( ata_trans->master_mbr.entry[tmp_counter].Sectors > 0 )
		{
			ata_data = (unsigned int *)malloc( ata_trans->master.bytes_per_sector );

			if( ata_data == NULL )
			{
				return( ATA_MALLLOC_ERROR | ata_result );
			}

			ata_result = ata_ide_read_sector_lba( ATA_MASTER_DRIVE, ata_trans->master_mbr.entry[tmp_counter].LBA, 1, ata_data );
#ifdef DEBUG
			printf("ata_ide_read_sector(FAT) result:%04X \n", ata_result );
#endif
			if( (ata_result &0xFF00) == ATA_OK	)
			{
				memcpy( &ata_bs_and_bpb, ata_data, sizeof(ata_bs_and_bpb) );
	
				memcpy( ata_trans->master_volume[tmp_counter].OEMName, ata_bs_and_bpb.BS_OEMName, sizeof(ata_bs_and_bpb.BS_OEMName) );

				if( (unsigned long)ata_bs_and_bpb.BPB_TotSec16 != 0 )
				{
					ata_trans->master_volume[tmp_counter].TotalSectors = ata_bs_and_bpb.BPB_TotSec16;
				}
				else
				{
					ata_trans->master_volume[tmp_counter].TotalSectors = ata_bs_and_bpb.BPB_TotSec32;
				}

				if( (unsigned long)ata_bs_and_bpb.BPB_FATSz16 != 0 )
				{
					ata_trans->master_volume[tmp_counter].FatSize = ata_bs_and_bpb.BPB_FATSz16;
				}
				else
				{
					ata_trans->master_volume[tmp_counter].FatSize = ata_bs_and_bpb.fat.fat32.BPB_FATSz32;
				}

				ata_trans->master_volume[tmp_counter].BytesPerSector = ata_bs_and_bpb.BPB_BytesPerSec;

				ata_trans->master_volume[tmp_counter].SectorsPerCluster = ata_bs_and_bpb.BPB_Sec_Per_clus; 

				ata_trans->master_volume[tmp_counter].ReservedSectors = ata_bs_and_bpb.BPB_RsvdSecCnt;					

				ata_trans->master_volume[tmp_counter].FirstVolumeSector = ata_trans->master_mbr.entry[0].LBA;

				ata_trans->master_volume[tmp_counter].RootDirSectors = (((ata_bs_and_bpb.BPB_RootEntCnt * 32) + (ata_bs_and_bpb.BPB_BytesPerSec - 1)) / ata_bs_and_bpb.BPB_BytesPerSec);

				ata_trans->master_volume[tmp_counter].FirstDataSector = ata_trans->master_volume[tmp_counter].ReservedSectors + (ata_bs_and_bpb.BPB_NumFATs * ata_trans->master_volume[tmp_counter].FatSize) + ata_trans->master_volume[tmp_counter].RootDirSectors;
				
   				ata_trans->master_volume[tmp_counter].DataSectors = ata_trans->master_volume[tmp_counter].TotalSectors - (ata_trans->master_volume[tmp_counter].ReservedSectors + (ata_bs_and_bpb.BPB_NumFATs * ata_trans->master_volume[tmp_counter].FatSize) + ata_trans->master_volume[tmp_counter].RootDirSectors);

				ata_trans->master_volume[tmp_counter].CountOfClusters =  ata_trans->master_volume[tmp_counter].DataSectors / ata_bs_and_bpb.BPB_Sec_Per_clus;

				if( ata_trans->master_volume[tmp_counter].CountOfClusters < 4085 )
				{
					ata_trans->master_volume[tmp_counter].FatType = FAT_12;
				}
				else if( ata_trans->master_volume[tmp_counter].CountOfClusters < 65525 )
				{
					ata_trans->master_volume[tmp_counter].FatType = FAT_16;
				}
				else
				{
					ata_trans->master_volume[tmp_counter].FatType = FAT_32;
				}

				ata_trans->master_volume[tmp_counter].SecondFatSector = ata_trans->master_mbr.entry[tmp_counter].LBA + ata_bs_and_bpb.BPB_RsvdSecCnt + ata_trans->master_volume[tmp_counter].FatSize;
				ata_trans->master_volume[tmp_counter].VolumeSize = ata_trans->master_volume[tmp_counter].TotalSectors * ata_trans->master_volume[tmp_counter].BytesPerSector;

				if( ata_trans->master_volume[tmp_counter].FatType == FAT_32 )
				{
					memcpy( ata_trans->master_volume[tmp_counter].VolumeName, ata_bs_and_bpb.fat.fat32.BS_VolLabel, sizeof(ata_bs_and_bpb.fat.fat32.BS_VolLabel) );
					ata_trans->master_volume[tmp_counter].VolumeID = ata_bs_and_bpb.fat.fat32.BS_VolID;
					ata_trans->master_volume[tmp_counter].FirstFatCluster = ata_bs_and_bpb.fat.fat32.BPB_RootClus;
					ata_trans->master_volume[tmp_counter].FirstRootDirSector =  ata_trans->master_volume[tmp_counter].FirstVolumeSector + FirstSectorOfCluster( ata_trans->master_volume[tmp_counter], ata_trans->master_volume[tmp_counter].FirstFatCluster);

				}
				else
				{
					memcpy( ata_trans->master_volume[tmp_counter].VolumeName, ata_bs_and_bpb.fat.fat16.BS_VolLabel, sizeof(ata_bs_and_bpb.fat.fat16.BS_VolLabel) );
					ata_trans->master_volume[tmp_counter].VolumeID = ata_bs_and_bpb.fat.fat16.BS_VolID;
					ata_trans->master_volume[tmp_counter].FirstFatSector = ata_trans->master_mbr.entry[tmp_counter].LBA + ata_bs_and_bpb.BPB_RsvdSecCnt;
					ata_trans->master_volume[tmp_counter].FirstRootDirSector = ata_trans->master_volume[tmp_counter].FirstVolumeSector + ata_trans->master_volume[tmp_counter].ReservedSectors + (ata_trans->master_volume[tmp_counter].FatSize * ata_bs_and_bpb.BPB_NumFATs);
				}
			}
			else
			{
				return( ATA_ERROR | ata_result );
			}
			
			free( ata_data );
		}
	}

	return( ATA_OK | ata_result );
}


