/*******************************************************************
 *
 *    DESCRIPTION:	fat.c
 *
 *    AUTHOR:		Mark de Jong
 *
 *    HISTORY:    
 *
 *******************************************************************/

/** include files **/
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "ata_if.h"

/** local definitions **/

/* default settings */

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

unsigned long FirstSectorOfCluster( volume_info_type volume_info, unsigned long cluster )
{
	unsigned long sector;

	sector = (( cluster -2 ) * volume_info.SectorsPerCluster) + volume_info.FirstDataSector;

	return( sector );
}

unsigned long ThisFATSectorNumber( volume_info_type volume_info, unsigned long cluster )
{
	unsigned long sector;
	unsigned long FATOffset;

	FATOffset = 0;

	if( volume_info.FatType == FAT_16 )
	{
		FATOffset = cluster * 2;
	}
	else if( volume_info.FatType == FAT_32 )
	{
		FATOffset = cluster * 4;
	}

	sector = volume_info.ReservedSectors + (FATOffset / volume_info.BytesPerSector );

	return( sector );
}

unsigned long ThisFATEntryOffset( volume_info_type volume_info, unsigned long cluster )
{
	unsigned long offset;
	unsigned long FATOffset;

	FATOffset = 0;

	if( volume_info.FatType == FAT_16 )
	{
		FATOffset = cluster * 2;
	}
	else if( volume_info.FatType == FAT_32 )
	{
		FATOffset = cluster * 4;
	}

	offset = ((FATOffset % volume_info.BytesPerSector ) / 2);

	return( offset );
}

unsigned long ClusterEntryValue( volume_info_type volume_info, unsigned long cluster )
{
	volatile unsigned long sector;
	volatile unsigned long offset;
	volatile unsigned long value;
	unsigned int  ata_data[512];
	volatile unsigned int  ata_result;

	value = 0;

	sector = volume_info.FirstVolumeSector + ThisFATSectorNumber( volume_info, cluster );

	offset = ThisFATEntryOffset( volume_info, cluster );

	ata_result = ata_ide_read_sector_lba( ATA_MASTER_DRIVE, sector, 1, ata_data );

	if( volume_info.FatType == FAT_16 )
	{
		value = (unsigned int)ata_data[ offset ];
	}
	else if( volume_info.FatType == FAT_32 )
	{
		value = (unsigned long) (*(unsigned long *)&ata_data[ offset ]) & 0x0FFFFFFFL;
	}
	
	return( value );
}

unsigned int ata_ide_read_cluster( volume_info_type volume_info, unsigned long cluster, unsigned int *ata_data )
{
	unsigned int  ata_sectors_per_cluster;
	unsigned long  ata_start_sector;
	unsigned int  ata_result = 0;

	ata_sectors_per_cluster = volume_info.SectorsPerCluster;
	ata_start_sector 	    = volume_info.FirstVolumeSector + FirstSectorOfCluster( volume_info, cluster );

	ata_result = ata_ide_read_sector_lba( ATA_MASTER_DRIVE, ata_start_sector, ata_sectors_per_cluster, ata_data );

	return( ata_result );
}